@@ -31,10 +31,11 @@
 #include "utils/StringUtils.hpp"
 #include "utils/utils.hpp"
 #include "voikko_defines.h"
+#include <cassert>
 
 using namespace libvoikko::morphology;
 using namespace libvoikko::character;
-using namespace std;
+using std::list;
 
 namespace libvoikko { namespace hyphenator {
 
@@ -66,6 +67,7 @@ char * AnalyzerToFinnishHyphenatorAdapter::hyphenate(const wchar_t * word, size_
 	if (hyphenations == 0) {
 		return 0;
 	}
+	assert(hyphenations[0]);
 	
 	int i = 0;
 	while (hyphenations[i] != 0) {
@@ -122,6 +124,7 @@ char * AnalyzerToFinnishHyphenatorAdapter::allPossibleHyphenPositions(const wcha
 	if (hyphenations == 0) {
 		return 0;
 	}
+	assert(hyphenations[0]);
 	
 	int i = 0;
 	while (hyphenations[i] != 0) {
@@ -169,7 +172,7 @@ char ** AnalyzerToFinnishHyphenatorAdapter::splitCompounds(const wchar_t * word,
 	}
 	size_t utf8Len = strlen(wordUtf8);
 	
-	list<Analysis *> * analyses = analyzer->analyze(wordUtf8);
+	list<Analysis *> * analyses = analyzer->analyze(wordUtf8, false);
 	
 	/* We may have to remove the trailing dot before hyphenation */
 	if (analyses->empty() && ignoreDot && len > 1 &&
@@ -178,7 +181,7 @@ char ** AnalyzerToFinnishHyphenatorAdapter::splitCompounds(const wchar_t * word,
 		utf8Len--;
 		*dotRemoved = true;
 		Analyzer::deleteAnalyses(analyses);
-		analyses = analyzer->analyze(wordUtf8);
+		analyses = analyzer->analyze(wordUtf8, false);
 	}
 	else {
 		*dotRemoved = false;
