@@ -36,7 +36,7 @@
 #include "spellchecker/HfstSpeller.hpp"
 #endif
 
-#ifdef HAVE_VFST
+#ifdef HAVE_EXPERIMENTAL_VFST
 #include "spellchecker/VfstSpeller.hpp"
 #endif
 
@@ -62,10 +62,10 @@ Speller * SpellerFactory::getSpeller(voikko_options_t * voikkoOptions,
 	}
 	#ifdef HAVE_HFST
 	if (spellBackend == "hfst") {
-		return new HfstSpeller(dictionary.getMorBackend().getPath());
+		return new HfstSpeller(dictionary.getMorBackend().getPath(), voikkoOptions);
 	}
 	#endif
-	#ifdef HAVE_VFST
+	#ifdef HAVE_EXPERIMENTAL_VFST
 	if (spellBackend == "vfst") {
 		return new VfstSpeller(dictionary.getMorBackend().getPath());
 	}
