@@ -1,19 +1,29 @@
-/* Libvoikko: Finnish spellchecker and hyphenator library
- * Copyright (C) 2009 - 2011 Harri Pitkänen <hatapitk@iki.fi>
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+/* The contents of this file are subject to the Mozilla Public License Version 
+ * 1.1 (the "License"); you may not use this file except in compliance with 
+ * the License. You may obtain a copy of the License at 
+ * http://www.mozilla.org/MPL/
+ * 
+ * Software distributed under the License is distributed on an "AS IS" basis,
+ * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+ * for the specific language governing rights and limitations under the
+ * License.
+ * 
+ * The Original Code is Libvoikko: Library of natural language processing tools.
+ * The Initial Developer of the Original Code is Harri Pitkänen <hatapitk@iki.fi>.
+ * Portions created by the Initial Developer are Copyright (C) 2009 - 2012
+ * the Initial Developer. All Rights Reserved.
+ * 
+ * Alternatively, the contents of this file may be used under the terms of
+ * either the GNU General Public License Version 2 or later (the "GPL"), or
+ * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+ * in which case the provisions of the GPL or the LGPL are applicable instead
+ * of those above. If you wish to allow use of your version of this file only
+ * under the terms of either the GPL or the LGPL, and not to allow others to
+ * use your version of this file under the terms of the MPL, indicate your
+ * decision by deleting the provisions above and replace them with the notice
+ * and other provisions required by the GPL or the LGPL. If you do not delete
+ * the provisions above, a recipient may use your version of this file under
+ * the terms of any one of the MPL, the GPL or the LGPL.
  *********************************************************************************/
 
 #include "spellchecker/suggestion/SuggestionStrategyTyping.hpp"
@@ -27,6 +37,7 @@
 #include "spellchecker/suggestion/SuggestionGeneratorSplitWord.hpp"
 #include "spellchecker/suggestion/SuggestionGeneratorSwap.hpp"
 #include "spellchecker/suggestion/SuggestionGeneratorVowelChange.hpp"
+#include "spellchecker/suggestion/SuggestionGeneratorDeleteTwo.hpp"
 
 using namespace std;
 
@@ -51,7 +62,8 @@ static const wchar_t * const REPLACEMENTS_2 =
 	L"1q" L"2q" L"2w" L"3w" L"3e"
 	L"4e" L"4r" L"5r" L"5t" L"6t"
 	L"6y" L"7y" L"7u" L"8u" L"8i"
-	L"9i" L"9o" L"0o" L"0p" L"+p";
+	L"9i" L"9o" L"0o" L"0p" L"+p"
+	L"ie";
 
 static const wchar_t * const REPLACEMENTS_3 =
 	L"es" L"sd" L"nh" L"uj" L"l\u00f6"
@@ -79,7 +91,14 @@ static const wchar_t * const REPLACEMENTS_4 =
 	L"vc" L"aw" L"az" L"sq";
 
 static const wchar_t * const REPLACEMENTS_5 =
-	L"ao" L"oa";
+	L"ao" L"oa" L"ou" L"tl" L"sr"
+	L"ai" L"e\u00e4" L"\u00e4e" L"uv" L"vu"
+	L"od" L"do" L"kq" L"pv" L"vp"
+	L"qe" L"eq" L"ad" L"da" L"rs"
+	L"et" L"te" L"ry" L"yr" L"tu"
+	L"ut" L"yi" L"iy" L"uo" L"ip"
+	L"pi" L"o\u00e5" L"hv" L"vh" L"hm"
+	L"mh";
 
 SuggestionStrategyTyping::SuggestionStrategyTyping(morphology::Analyzer * morAnalyzer, size_t maxCost) : SuggestionStrategy(maxCost) {
 	primaryGenerators.push_back(
@@ -117,6 +136,8 @@ SuggestionStrategyTyping::SuggestionStrategyTyping(morphology::Analyzer * morAna
 		new SuggestionGeneratorReplaceTwo(REPLACEMENTS_3, morAnalyzer));
 	generators.push_back(
 		new SuggestionGeneratorReplaceTwo(REPLACEMENTS_4, morAnalyzer));
+	generators.push_back(
+		new SuggestionGeneratorDeleteTwo(morAnalyzer));
 	generators.push_back(
 		new SuggestionGeneratorReplacement(REPLACEMENTS_5, morAnalyzer));
 }
